@@ -1,4 +1,4 @@
-"""If the following ENV VAR are available, many end-to-end test cases would run.
+"""If the following ENV VAR were available, many end-to-end test cases would run.
 LAB_APP_CLIENT_SECRET=...
 LAB_OBO_CLIENT_SECRET=...
 LAB_APP_CLIENT_ID=...
@@ -27,10 +27,23 @@ import requests
 import msal
 from tests.http_client import MinimalHttpClient, MinimalResponse
 from msal.oauth2cli import AuthCodeReceiver
+from msal.oauth2cli.oidc import decode_part
 
+try:
+    import pymsalruntime
+    broker_available = True
+except ImportError:
+    broker_available = False
 logger = logging.getLogger(__name__)
 logging.basicConfig(level=logging.DEBUG if "-v" in sys.argv else logging.INFO)
 
+try:
+    from dotenv import load_dotenv  # Use this only in local dev machine
+    load_dotenv()  # take environment variables from .env.
+except ImportError:
+    logger.warn("Run pip install -r requirements.txt for optional dependency")
+
+_AZURE_CLI = "04b07795-8ddb-461a-bbee-02f9e1bf7b46"
 
 def _get_app_and_auth_code(
         client_id,
@@ -93,7 +106,7 @@ class E2eTestCase(unittest.TestCase):
             assertion()
 
     def assertCacheWorksForUser(
-            self, result_from_wire, scope, username=None, data=None):
+            self, result_from_wire, scope, username=None, data=None, auth_scheme=None):
         logger.debug(
             "%s: cache = %s, id_token_claims = %s",
             self.id(),
@@ -109,35 +122,34 @@ class E2eTestCase(unittest.TestCase):
                 set(scope) <= set(result_from_wire["scope"].split(" "))
                 ):
             # Going to test acquire_token_silent(...) to locate an AT from cache
-            result_from_cache = self.app.acquire_token_silent(
-                scope, account=account, data=data or {})
-            self.assertIsNotNone(result_from_cache)
+            silent_result = self.app.acquire_token_silent(
+                scope, account=account, data=data or {}, auth_scheme=auth_scheme)
+            self.assertIsNotNone(silent_result)
             self.assertIsNone(
-                result_from_cache.get("refresh_token"), "A cache hit returns no RT")
-            self.assertEqual(
-                result_from_wire['access_token'], result_from_cache['access_token'],
-                "We should get a cached AT")
+                silent_result.get("refresh_token"), "acquire_token_silent() should return no RT")
+            if auth_scheme:
+                self.assertNotEqual(
+                    self.app._TOKEN_SOURCE_CACHE, silent_result[self.app._TOKEN_SOURCE])
+            else:
+                self.assertEqual(
+                    self.app._TOKEN_SOURCE_CACHE, silent_result[self.app._TOKEN_SOURCE])
 
         if "refresh_token" in result_from_wire:
+            assert auth_scheme is None
             # Going to test acquire_token_silent(...) to obtain an AT by a RT from cache
             self.app.token_cache._cache["AccessToken"] = {}  # A hacky way to clear ATs
-        result_from_cache = self.app.acquire_token_silent(
-            scope, account=account, data=data or {})
-        if "refresh_token" not in result_from_wire:
-            self.assertEqual(
-                result_from_cache["access_token"], result_from_wire["access_token"],
-                "The previously cached AT should be returned")
-        self.assertIsNotNone(result_from_cache,
+            silent_result = self.app.acquire_token_silent(
+                scope, account=account, data=data or {})
+            self.assertIsNotNone(silent_result,
                 "We should get a result from acquire_token_silent(...) call")
-        self.assertIsNotNone(
-            # We used to assert it this way:
-            #   result_from_wire['access_token'] != result_from_cache['access_token']
-            # but ROPC in B2C tends to return the same AT we obtained seconds ago.
-            # Now looking back, "refresh_token grant would return a brand new AT"
-            # was just an empirical observation but never a commitment in specs,
-            # so we adjust our way to assert here.
-            (result_from_cache or {}).get("access_token"),
-            "We should get an AT from acquire_token_silent(...) call")
+            self.assertEqual(
+                # We used to assert it this way:
+                #   result_from_wire['access_token'] != silent_result['access_token']
+                # but ROPC in B2C tends to return the same AT we obtained seconds ago.
+                # Now looking back, "refresh_token grant would return a brand new AT"
+                # was just an empirical observation but never a commitment in specs,
+                # so we adjust our way to assert here.
+                self.app._TOKEN_SOURCE_IDP, silent_result[self.app._TOKEN_SOURCE])
 
     def assertCacheWorksForApp(self, result_from_wire, scope):
         logger.debug(
@@ -150,11 +162,9 @@ class E2eTestCase(unittest.TestCase):
             self.app.acquire_token_silent(scope, account=None),
             "acquire_token_silent(..., account=None) shall always return None")
         # Going to test acquire_token_for_client(...) to locate an AT from cache
-        result_from_cache = self.app.acquire_token_for_client(scope)
-        self.assertIsNotNone(result_from_cache)
-        self.assertEqual(
-            result_from_wire['access_token'], result_from_cache['access_token'],
-            "We should get a cached AT")
+        silent_result = self.app.acquire_token_for_client(scope)
+        self.assertIsNotNone(silent_result)
+        self.assertEqual(self.app._TOKEN_SOURCE_CACHE, silent_result[self.app._TOKEN_SOURCE])
 
     @classmethod
     def _build_app(cls,
@@ -192,6 +202,7 @@ class E2eTestCase(unittest.TestCase):
             client_secret=None,  # Since MSAL 1.11, confidential client has ROPC too
             azure_region=None,
             http_client=None,
+            auth_scheme=None,
             **ignored):
         assert authority and client_id and username and password and scope
         self.app = self._build_app(
@@ -203,12 +214,14 @@ class E2eTestCase(unittest.TestCase):
         self.assertEqual(
             self.app.get_accounts(username=username), [], "Cache starts empty")
         result = self.app.acquire_token_by_username_password(
-            username, password, scopes=scope)
+            username, password, scopes=scope, auth_scheme=auth_scheme)
         self.assertLoosely(result)
         self.assertCacheWorksForUser(
             result, scope,
             username=username,  # Our implementation works even when "profile" scope was not requested, or when profile claims is unavailable in B2C
+            auth_scheme=auth_scheme,
             )
+        return result
 
     @unittest.skipIf(
         os.getenv("TRAVIS"),  # It is set when running on TravisCI or Github Actions
@@ -246,6 +259,7 @@ class E2eTestCase(unittest.TestCase):
             data=None,  # Needed by ssh-cert feature
             prompt=None,
             enable_msa_passthrough=None,
+            auth_scheme=None,
             **ignored):
         assert client_id and authority and scope
         self.app = self._build_app(client_id, authority=authority)
@@ -266,6 +280,7 @@ class E2eTestCase(unittest.TestCase):
     </ol></body></html>""".format(id=self.id(), hint=_get_hint(
                 html_mode=True,
                 username=username, lab_name=lab_name, username_uri=username_uri)),
+            auth_scheme=auth_scheme,
             data=data or {},
             )
         self.assertIn(
@@ -279,7 +294,8 @@ class E2eTestCase(unittest.TestCase):
                 username, result["id_token_claims"]["preferred_username"],
                 "You are expected to sign in as account {}, but tokens returned is for {}".format(
                     username, result["id_token_claims"]["preferred_username"]))
-        self.assertCacheWorksForUser(result, scope, username=None, data=data or {})
+        self.assertCacheWorksForUser(
+            result, scope, username=None, data=data or {}, auth_scheme=auth_scheme)
         return result  # For further testing
 
 
@@ -1147,5 +1163,117 @@ class ArlingtonCloudTestCase(LabBasedTestCase):
         #       If this test case passes without exception,
         #       it means MSAL Python is not affected by that.
 
+
+@unittest.skipUnless(broker_available, "AT POP feature is only supported by using broker")
+class PopTestCase(LabBasedTestCase):
+    def test_at_pop_should_contain_pop_scheme_content(self):
+        auth_scheme = msal.PopAuthScheme(
+            http_method=msal.PopAuthScheme.HTTP_GET,
+            url="https://www.Contoso.com/Path1/Path2?queryParam1=a&queryParam2=b",
+            nonce="placeholder",
+            )
+        result = self._test_acquire_token_interactive(
+            # Lab test users tend to get kicked out from WAM, we use local user to test
+            client_id=_AZURE_CLI,
+            authority="https://login.microsoftonline.com/organizations",
+            scope=["https://management.azure.com/.default"],
+            auth_scheme=auth_scheme,
+            )   # It also tests assertCacheWorksForUser()
+        self.assertEqual(result["token_source"], "broker", "POP is only supported by broker")
+        self.assertEqual(result["token_type"], "pop")
+        payload = json.loads(decode_part(result["access_token"].split(".")[1]))
+        logger.debug("AT POP payload = %s", json.dumps(payload, indent=2))
+        self.assertEqual(payload["m"], auth_scheme._http_method)
+        self.assertEqual(payload["u"], auth_scheme._url.netloc)
+        self.assertEqual(payload["p"], auth_scheme._url.path)
+        self.assertEqual(payload["nonce"], auth_scheme._nonce)
+
+    # TODO: Remove this, as ROPC support is removed by Broker-on-Win
+    def test_at_pop_via_testingsts_service(self):
+        """Based on https://testingsts.azurewebsites.net/ServerNonce"""
+        self.skipTest("ROPC support is removed by Broker-on-Win")
+        auth_scheme = msal.PopAuthScheme(
+            http_method="POST",
+            url="https://www.Contoso.com/Path1/Path2?queryParam1=a&queryParam2=b",
+            nonce=requests.get(
+                # TODO: Could use ".../missing" and then parse its WWW-Authenticate header
+                "https://testingsts.azurewebsites.net/servernonce/get").text,
+            )
+        config = self.get_lab_user(usertype="cloud")
+        config["password"] = self.get_lab_user_secret(config["lab_name"])
+        result = self._test_username_password(auth_scheme=auth_scheme, **config)
+        self.assertEqual(result["token_type"], "pop")
+        shr = result["access_token"]
+        payload = json.loads(decode_part(result["access_token"].split(".")[1]))
+        logger.debug("AT POP payload = %s", json.dumps(payload, indent=2))
+        self.assertEqual(payload["m"], auth_scheme._http_method)
+        self.assertEqual(payload["u"], auth_scheme._url.netloc)
+        self.assertEqual(payload["p"], auth_scheme._url.path)
+        self.assertEqual(payload["nonce"], auth_scheme._nonce)
+
+        validation = requests.post(
+            # TODO: This endpoint does not seem to validate the url
+            "https://testingsts.azurewebsites.net/servernonce/validateshr",
+            data={"SHR": shr},
+            )
+        self.assertEqual(validation.status_code, 200)
+
+    def test_at_pop_calling_pattern(self):
+        # The calling pattern was described here:
+        # https://identitydivision.visualstudio.com/DevEx/_git/AuthLibrariesApiReview?path=/PoPTokensProtocol/PoP_API_In_MSAL.md&_a=preview&anchor=proposal-2---optional-isproofofposessionsupportedbyclient-helper-(accepted)
+
+        # It is supposed to call app.is_pop_supported() first,
+        # and then fallback to bearer token code path.
+        # We skip it here because this test case has not yet initialize self.app
+        # assert self.app.is_pop_supported()
+        api_endpoint = "https://20.190.132.47/beta/me"
+        resp = requests.get(api_endpoint, verify=False)
+        self.assertEqual(resp.status_code, 401, "Initial call should end with an http 401 error")
+        result = self._get_shr_pop(**dict(
+            self.get_lab_user(usertype="cloud"),  # This is generally not the current laptop's default AAD account
+            scope=["https://graph.microsoft.com/.default"],
+            auth_scheme=msal.PopAuthScheme(
+                http_method=msal.PopAuthScheme.HTTP_GET,
+                url=api_endpoint,
+                nonce=self._extract_pop_nonce(resp.headers.get("WWW-Authenticate")),
+                ),
+            ))
+        resp = requests.get(api_endpoint, verify=False, headers={
+            "Authorization": "pop {}".format(result["access_token"]),
+            })
+        if resp.status_code != 200:
+            # TODO https://teams.microsoft.com/l/message/19:b1697a70b1de43ddaea281d98ff2e985@thread.v2/1700184847801?context=%7B%22contextType%22%3A%22chat%22%7D
+            self.skipTest("We haven't got this end-to-end test case working")
+        self.assertEqual(resp.status_code, 200, "POP resource should be accessible")
+
+    def _extract_pop_nonce(self, www_authenticate):
+        # This is a hack for testing purpose only. Do not use this in prod.
+        # FYI: There is a www-authenticate package but it falters when encountering realm=""
+        import re
+        found = re.search(r'nonce="(.+?)"', www_authenticate)
+        if found:
+            return found.group(1)
+
+    def _get_shr_pop(
+            self, client_id=None, authority=None, scope=None, auth_scheme=None,
+            **kwargs):
+        result = self._test_acquire_token_interactive(
+            # Lab test users tend to get kicked out from WAM, we use local user to test
+            client_id=client_id,
+            authority=authority,
+            scope=scope,
+            auth_scheme=auth_scheme,
+            **kwargs)  # It also tests assertCacheWorksForUser()
+        self.assertEqual(result["token_source"], "broker", "POP is only supported by broker")
+        self.assertEqual(result["token_type"], "pop")
+        payload = json.loads(decode_part(result["access_token"].split(".")[1]))
+        logger.debug("AT POP payload = %s", json.dumps(payload, indent=2))
+        self.assertEqual(payload["m"], auth_scheme._http_method)
+        self.assertEqual(payload["u"], auth_scheme._url.netloc)
+        self.assertEqual(payload["p"], auth_scheme._url.path)
+        self.assertEqual(payload["nonce"], auth_scheme._nonce)
+        return result
+
+
 if __name__ == "__main__":
     unittest.main()
